import java.io.*;
import java.sql.*;
import javax.security.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * Architecture client maintains a private key clientsk and we 
 * have the the public key of the client at our end, we decrypt
 * the Message with the private key of Server, and crypt the 
 * outgoing message with the public key of the client, the public
 * key is chosen according to the clientID of application
 *
 * @author sandy
 * @date 13/11/99
 * @vendor SeSe Technologies :)
 */


public class CipherServer {
  protected static final String serverID = "SeSe SERVER version 1.0.0",
  keyPairName = "RSA",
  cipherName   = "RSA/ECB/OneAndZeroes";
  String clientsPkFile = "serverPublickey";   // server's public key stored here 
  String clientsSkFile = "serverSecretKey";   // server's secret key stored here

  protected static final int strength = 512;

  protected static Connection conn =null;
  public  String clientID =null;
  protected Cipher rsa =null;
  protected PublicKey clientPk =null;
  protected PrivateKey serverSk =null;

  // Opening a Single time connection to SQL database.
  static {
    try {
      // This will help in changing database engine depending on configuration
      // files
      Class.forName("oracle.jdbc.driver.OracleDriver").newInstance();
      final String url= "jdbc:oracle:thin:@everest.cse.iitb.ernet.in:1521:GEN";
      conn = DriverManager.getConnection(url, "userinfo", "fighter");
    } catch (Exception e) { 
      e.printStackTrace(); 
    }
  }
  
  /*
   * This Method converts byte array into an InputStream
   */
  protected static ObjectInputStream getObjectInputStream(byte [] inbuf)
    throws IOException {
      ByteArrayInputStream bais = new ByteArrayInputStream(inbuf);
      return new ObjectInputStream(bais);
    }

  /**
   * This method loads Client Public Key from Database
   */
  public boolean loadClientDBPk() {
    if (clientPk != null ) return true;
    try {
      Statement stmt3 = conn.createStatement();
      ResultSet rs3 = stmt3.executeQuery("select pub_key from application where app_id = '" + clientID + "'");
      if (rs3.next()) {
	ObjectInputStream ois = getObjectInputStream(rs3.getBytes(1));
	clientPk = (PublicKey) ois.readObject();
      } else {
	return false; // application id doesn't exists in our database.
      }
      rs3.close();
      stmt3.close();
      return true;
    }
    catch ( Exception anyex ) { 
      anyex.printStackTrace();
    }
    return false;
  }

  /**
   * This method loads Client PublicKey from the file system
   */
  public boolean loadClientPk() {
    if (clientPk!=null) return true;
    try {
      FileInputStream fis=new FileInputStream("/users/ug97/bhalotia/tmp/"+clientID);
      ObjectInputStream ois=new ObjectInputStream(fis);
      clientPk = (PublicKey) ois.readObject();
    } catch (Exception e) { 
      e.printStackTrace();
      return false; 
    }
    return true;
  }

  /**
   * This method when given a public key for a client as a byte array
   * puts it into the Server Database as a Binary Large Object( blob)
   */
  public static boolean putClientPublicKey(PrintWriter out, String clientID, byte[] tmp) {
    out.println("<PRE>");
    out.println(tmp.length);
    try {
      PreparedStatement stmt2 = conn.prepareStatement("update application set pub_key = ? " +
	  "where app_id = '" + clientID + "'");
      stmt2.setBytes(1,tmp);
      int cnt1=stmt2.executeUpdate();
      if (cnt1!=1) throw new Exception("No such user");
      stmt2.close();
      out.println("</PRE>");
      return true;
    }
    catch ( Exception anyex ) {
      anyex.printStackTrace(out);
    }
    out.println("</PRE>");
    return false;
  }

  public String getLogin() { 
    return clientID; 
  }

  //                       
  // then performs serversk  (-) to get the resulting Object, login
  //

  /**
   * This Method decrypts the request stream using the server's secret key,
   * deserializes it and returns an object associated with it
   */
  public Object getDecryptedObject(HttpServletRequest req) throws Exception {
      int inlen=req.getContentLength();
      byte [] tmp=new byte[inlen];
      req.getInputStream().read(tmp);
      rsa = Cipher.getInstance(cipherName);
      rsa.initDecrypt(serverSk);
      byte [] combo = rsa.crypt(tmp);
      ObjectInputStream ois = getObjectInputStream(combo);
      clientID = (String)ois.readObject();
      Object ret=ois.readObject();
      ois.close();
    return ret;
  }

  // added by sandy
  // returns serversk[serverLogin, Object]
  /**
   * Takes an object, serializes it and returns a byte array crypted by
   * the client's public key.
   */
  public byte [] getCryptedStream(Object ob) throws Exception {
      ByteArrayOutputStream baos = new ByteArrayOutputStream();
      ObjectOutputStream oos = new ObjectOutputStream(baos);
      oos.writeObject(serverID);
      oos.writeObject(ob);
      oos.close();
      byte [] plain = baos.toByteArray();
      if (!loadClientPk()) throw new Exception("Unable to load client's Public Key");
      rsa = Cipher.getInstance(cipherName);
      rsa.initEncrypt(clientPk);
      return rsa.crypt(plain);
  }

  /**
   * Loads the server secret key.
   */
  public CipherServer(String serverSkFile) {
    try {
      FileInputStream   fl   =new FileInputStream(serverSkFile);
      ObjectInputStream ois  =new ObjectInputStream(fl);
      serverSk          =(PrivateKey)ois.readObject();
      fl.close();
      ois.close();
    }
    catch ( Exception anyex ) {
      anyex.printStackTrace();
    }
  }

  /**
   * On command line invocation creates server public key and server
   * private key.Server Public key should be accessible to application
   * developers from the servers site.And private key should be stored
   * securely.
   */
  public static void main(String args[]) {
    try {
      System.out.println("CipherServer: generating Public and Private Key for Server");
      KeyPairGenerator kg = KeyPairGenerator.getInstance(keyPairName);
      kg.initialize(strength, new SecureRandom());
      KeyPair pair = kg.generateKeyPair();
      PublicKey spk = pair.getPublic();
      PrivateKey ssk = pair.getPrivate();
      FileOutputStream   fl  =new FileOutputStream("serverPublicKey");
      ObjectOutputStream oos =new ObjectOutputStream(fl);
      oos.writeObject(spk);
      fl                     =new FileOutputStream("serverSecretKey");
      oos                    =new ObjectOutputStream(fl);
      oos.writeObject(ssk);
      System.out.println("CipherServer: Please store the serverPublicKey at a secure place"
	  +" and broadcast the public key at your site"); 
    } catch (Exception e) {
      e.printStackTrace();
      System.err.println("CipherServer: Unable to generate the server public and private"
	  +"keys");
    }
  }
}
