import java.io.*;
import java.util.*;
import javax.servlet.*;

/**
 * This class has been dowloaded from Oreilly's web site, please
 * visit their site for documentation.
 */
public class MultipartRequest {
	private static final int DEFAULT_MAX_POST_SIZE=1024 * 1024;
	private ServletRequest req;
	private File dir;
	private int maxSize;
	public byte [] key;

	private Hashtable parameters = new Hashtable();
	private Hashtable files = new Hashtable();

	public MultipartRequest(ServletRequest request,String saveDirectory)
			throws IOException{
		this(request,saveDirectory,DEFAULT_MAX_POST_SIZE);
	}

	public MultipartRequest(ServletRequest request,String saveDirectory,
			int maxPostSize) throws IOException {
		if(request==null)
			throw new IllegalArgumentException("request cannot null");
		if(saveDirectory == null)
			throw new IllegalArgumentException("saveDirectory cannot be null");
		if(maxPostSize <=0)
			throw new IllegalArgumentException("maxPostSize must be positive");

		req=request;
		dir=new File(saveDirectory);

		maxSize = maxPostSize;
		boolean fa=false;
		fa=dir.isDirectory();
		if( !fa )
			throw new IllegalArgumentException("Not a directory:" +saveDirectory);
		fa=dir.canWrite();
		if( !fa )
			throw new IllegalArgumentException("Not writable"+ saveDirectory);
		readRequest();
	}

	public Enumeration getParameterNames() {
		return parameters.keys();
	}

	public Enumeration getFileNames() {
		return files.keys();
	}

	public String getParameter(String name) {
		try{
			String param = (String) parameters.get(name);
			if(param.equals(" ")) 
				return null;
			return param;
		}
		catch(Exception e){
			return null;
		}
	}

	
	public String getFilesystemName(String name) {
		try {
			UploadedFile file = (UploadedFile) files.get(name);
			return file.getFilesystemName();
		}
		catch(Exception e){
			return null;
		}
	}

	public String getContentType(String name){
		try{
			UploadedFile file =(UploadedFile)files.get(name);
			return file.getContentType();
		}
		catch(Exception e){
			return null;
		}
	}

	public File getFile(String name){
		try {
			UploadedFile file=(UploadedFile) files.get(name);
			return file.getFile();
		}
		catch (Exception e){
			return null;
		}
	}

	protected void readRequest() throws IOException {
		String type = req.getContentType();
		if(type==null ||
			!type.toLowerCase().startsWith("multipart/form-data")){

			throw new IOException("Posted content type isn't multipart/form-data");
		}
		int length = req.getContentLength();
		if(length > maxSize){
			throw new IOException("Posted content length of"+ length+" exceeds limit of "+maxSize);
		}

		String boundary= extractBoundary(type);
		if(boundary == null){
			throw new IOException("Separation boundary was not specified.");
		}

		MultipartInputStreamHandler in =new MultipartInputStreamHandler
			(req.getInputStream(),boundary,length);

		String line = in.readLine();
		if(line == null) {
			throw new IOException("Corrupt form data: premature ending");
		}

		if(!line.startsWith(boundary)) {
			throw new IOException("Corrupt form data: no leading boundary");
		}

		boolean done = false;
		while(!done){
			done=readNextPart(in,boundary);
		}
	}

	protected boolean readNextPart(MultipartInputStreamHandler in,
			String boundary) throws IOException{
		String line =in.readLine();
		if(line == null) {
			return true;
		}

		String[] dispInfo = extractDispositionInfo(line);
		String disposition = dispInfo[0];
		String name =dispInfo[1];
		String filename =dispInfo[2];

		line = in.readLine();
		if(line == null){
			return true;
		}

		String contentType = extractContentType(line);
		if(contentType != null) {
			line=in.readLine();
			if(line==null || line.length()>0){
				throw new IOException("Malformed line after content type:"+line);
			}
		}
		else {
			contentType = "application/octet-stream";
		}

		if(filename == null){
			String value = readParameter(in,boundary);
			parameters.put(name,value);
		}
		else {
			readAndSaveFile(in,boundary,filename);
			if(filename.equals("unknown")) {
			     files.put(name,new UploadedFile(null,null,null));
			}
			else {
				files.put(name,new UploadedFile(dir.toString(),						filename,contentType));
			}
		}
		return false;
	}

	protected String readParameter(MultipartInputStreamHandler in,
			String boundary) throws IOException {
		StringBuffer sbuf = new StringBuffer();
		String line;

		while((line = in.readLine()) != null) {
			if(line.startsWith(boundary))
				break;
			sbuf.append(line+"\r\n");
		}

		if(sbuf.length() == 0) {
			return null;
		}

		sbuf.setLength(sbuf.length() -2) ;
		return sbuf.toString();
	}

	protected void readAndSaveFile(MultipartInputStreamHandler in,
			String boundary,String filename) throws IOException{
		//File f =new File(dir + File.separator +filename);
		//FileOutputStream fos =new FileOutputStream(f);
		ByteArrayOutputStream fos=new ByteArrayOutputStream();
		BufferedOutputStream out =new BufferedOutputStream(fos,8* 1024);
		byte[] bbuf =new byte[8 * 1024];
		int result;
		String line;
/*		//from here
		String form_no="";
		
      	Enumeration params = this.getParameterNames();
      	while (params.hasMoreElements()) {
        	String name = (String)params.nextElement();
        	String value = this.getParameter(name);
			if (name.startsWith("formno"))
			{
				form_no =value;
			}
		}
		insertFile infile = new insertFile(form_no);
		infile.Set_Connection();
		boolean rnflag = false;
		while(( result= in.readLine(bbuf,0,bbuf.length)) != -1) {
			if(result>2 && bbuf[0] == '-' && bbuf[1] == '-') {
				line= new String(bbuf,0,result,"ISO-8859-1");
				if(line.startsWith(boundary)) {
					break;
				}
			}
			
			if(rnflag) {
				out.write('\r');
				out.write('\n');
				rnflag= false;
			}

			if( result>=2 && bbuf[result-2] =='\r' &&
					bbuf[result-1] == '\n') {
				out.write(bbuf,0,result - 2);
				rnflag = true;
			}
			else {
				out.write(bbuf,0,result);
				infile.insertString(new String(bbuf,0,result));
			}
		}*/

		//ServletInputStream.readLine() has the annoting habit
		//of adding a \r\n to the end of the last line.
		//Since we want a byte for byte transfer we have to cut
		//those gaps

		boolean rnflag = false;
		while((result = in.readLine(bbuf, 0, bbuf.length))!= -1) {
		  // check for boundary
		  if (result > 2 && bbuf[0] == '-' && bbuf[1] =='-') {
		    // quick pre-check
		    line = new String (bbuf, 0, result, "ISO-8859-1");
		    if(line.startsWith(boundary)) break;
		  }
		  //Are we suposed to write \r\n for the last iteration?
		  if(rnflag) {
		    out.write('\r'); out.write('\n');
		    rnflag = false;
		  }
		  // Write teh buffer, postpone any ending \r\n
		  if ( result >=2 &&
		      bbuf[result - 2] == '\r' &&
		      bbuf[result - 1] == '\n')  {
		     out.write(bbuf, 0, result - 2); //skip the last 2 chars
		     rnflag = true; //make a note to write them in the nest iteration
		  }
		  else {
		    out.write(bbuf, 0, result);
		  }
		}
		out.flush();
		out.close();
		fos.close();
		key=fos.toByteArray();
	}

			
	private String extractBoundary(String line){
		int index = line.indexOf("boundary=");
		if(index == -1) {
			return null ;
		}

		String boundary = line.substring(index + 9);
		boundary = "--" + boundary;
		return boundary;
	}

	private String[] extractDispositionInfo(String line) throws IOException{		
		
		
		String[] retval = new String[3];

		String origline = line;
		line=origline.toLowerCase();

		int start = line.indexOf("content-disposition: ");
		int end = line.indexOf(";");

		if(start == -1 || end == -1) {
			throw new IOException("contContent disposition corrupt: " +origline);	
		}

		String disposition = line.substring(start + 21,end);
		if( !disposition.equals("form-data")) {
			throw new IOException("Invalid content disposition: " +disposition);
		}

		start = line.indexOf("name=\"",end);
		end = line.indexOf("\"",start+7);
		if(start == -1 || end == -1) {
			throw new IOException("nameContent disposition corrupt: " +origline);
		}

		String name= origline.substring(start + 6,end);

		String filename = null;
		start = line.indexOf("filename=\"",end + 2);
		end = line.indexOf("\"",start + 10);

		if(start != -1 && end != -1) {
			filename = origline.substring(start + 10,end);
			int slash= Math.max(filename.lastIndexOf('/'),
					filename.lastIndexOf('\\'));
			if(slash > -1)
				filename=filename.substring(slash + 1);
			if(filename.equals(""))
				filename = "unknown";
		}
		retval[0]=disposition;
		retval[1]=name;
		retval[2]=filename;
		return retval;
	}

	private String extractContentType(String line) throws IOException {
		String contentType = null;
		String origline = line;
		line = origline.toLowerCase();

		if(line.startsWith("content-type")) {
			int start = line.indexOf(" ");
			if(start == -1) {
				throw new IOException("Content type corrupt: " +origline);
			}
			contentType = line.substring(start + 1);
		}
		else if(line.length() != 0) {
			throw new IOException("Malformed line after disposition"				+origline);
		}

		return contentType;
	}
}





class UploadedFile {
	private String dir;
	private String filename;
	private String type;

	UploadedFile(String dir,String filename,String type){
		this.dir=dir;
		this.filename=filename;
		this.type=type;
	}

	public String getContentType(){
		return type;
	}

	public String getFilesystemName() {
		return filename;
	}

	public File getFile() {
		if(dir == null || filename == null )
			return null;
		else{
			String path=new String(dir+"/"+filename);
			
			//return new File(path);
			//return new File(dir + "/" + filename);
			//return new File(dir + File.separator + filename);
			//File fn=new File( dir + File.separator + filename);
			File fn =new File(dir + File.separator +filename);
			return fn;
		}
	}
}


class MultipartInputStreamHandler {
	ServletInputStream in;
	String boundary;
	int totalExpected;
	int totalRead = 0;
	byte[] buf = new byte[8 * 1024];

	public MultipartInputStreamHandler(ServletInputStream in,
			String boundary,int totalExpected) {
		this.in = in;
		this.boundary=boundary;
		this.totalExpected=totalExpected;
	}

	public String readLine() throws IOException {
		StringBuffer sbuf = new StringBuffer();
		int result;
		String line;

		do {
			result = this.readLine(buf,0,buf.length);
			if(result != -1)
			sbuf.append(new String(buf,0,result,"ISO-8859-1"));
		} while(result == buf.length);
		if(sbuf.length() == 0) {
			return null;
		}

		sbuf.setLength(sbuf.length() -2);
		return sbuf.toString();
	}

	public int readLine(byte b[], int off,int len) throws IOException {
		if(totalRead >= totalExpected) {	
			return -1;
		}
		else {
			int result = in.readLine(b,off,len);
			if(result > 0) 
				totalRead += result;
			return result;
		}
	}
}
	











